<?php
/**
* @package: phpBB 3.0.8 :: 
* @version: $Id: booksearch.php, v0.0.8 2011-08-26 11:08:26 leviatan21 Exp $
* @copyright: leviatan21 < info@mssti.com > (Gabriel) http://www.mssti.com/phpbb3/
* @license: http://opensource.org/licenses/gpl-license.php GNU Public License 
* @author: leviatan21 - http://www.phpbb.com/community/memberlist.php?mode=viewprofile&u=345763
**/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* @ignore
*/

/**
* fulltext_native
* phpBB's own db driven fulltext search, version 2
* @package search
*/
class booksearch extends fulltext_native
{
	/**
	* This function fills $this->search_query with the cleaned user search query.
	*
	* If $terms is 'any' then the words will be extracted from the search query
	* and combined with | inside brackets. They will afterwards be treated like
	* an standard search query.
	*
	* Then it analyses the query and fills the internal arrays $must_not_contain_ids,
	* $must_contain_ids and $must_exclude_one_ids which are later used by keyword_search().
	*
	* @param	string	$keywords	contains the search query string as entered by the user
	* @param	string	$terms		is either 'all' (use search query as entered, default words to 'must be contained in post')
	* 	or 'any' (find all posts containing at least one of the given words)
	* @return	boolean				false if no valid keywords were found and otherwise true
	*
	* @access	public
	*/
	function split_keywords_book($keywords, $terms)
	{
		global $db, $user, $config;

	//	$tokens = '+-|()*';
		$tokens = '+-|()';

		$keywords = trim($this->cleanup($keywords, $tokens));

		// allow word|word|word without brackets
		if ((strpos($keywords, ' ') === false) && (strpos($keywords, '|') !== false) && (strpos($keywords, '(') === false))
		{
			$keywords = '(' . $keywords . ')';
		}

		$open_bracket = $space = false;
		for ($i = 0, $n = strlen($keywords); $i < $n; $i++)
		{
			if ($open_bracket !== false)
			{
				switch ($keywords[$i])
				{
					case ')':
						if ($open_bracket + 1 == $i)
						{
							$keywords[$i - 1] = '|';
							$keywords[$i] = '|';
						}
						$open_bracket = false;
					break;
					case '(':
						$keywords[$i] = '|';
					break;
					case '+':
					case '-':
					case ' ':
						$keywords[$i] = '|';
					break;
					case '*':
						if ($i === 0 || ($keywords[$i - 1] !== '*' && strcspn($keywords[$i - 1], $tokens) === 0))
						{
							if ($i === $n - 1 || ($keywords[$i + 1] !== '*' && strcspn($keywords[$i + 1], $tokens) === 0))
							{
								$keywords = substr($keywords, 0, $i) . substr($keywords, $i + 1);
							}
						}
					break;
				}
			}
			else
			{
				switch ($keywords[$i])
				{
					case ')':
						$keywords[$i] = ' ';
					break;
					case '(':
						$open_bracket = $i;
						$space = false;
					break;
					case '|':
						$keywords[$i] = ' ';
					break;
					case '-':
					case '+':
						$space = $keywords[$i];
					break;
					case ' ':
						if ($space !== false)
						{
							$keywords[$i] = $space;
						}
					break;
					default:
						$space = false;
				}
			}
		}

		if ($open_bracket)
		{
			$keywords .= ')';
		}

		$match = array(
			'#  +#',
			'#\|\|+#',
			'#(\+|\-)(?:\+|\-)+#',
			'#\(\|#',
			'#\|\)#',
		);
		$replace = array(
			' ',
			'|',
			'$1',
			'(',
			')',
		);

		$keywords = preg_replace($match, $replace, $keywords);
		$num_keywords = sizeof(explode(' ', $keywords));

		// We limit the number of allowed keywords to minimize load on the database
		if ($config['max_num_search_keywords'] && $num_keywords > $config['max_num_search_keywords'])
		{
			trigger_error($user->lang('MAX_NUM_SEARCH_KEYWORDS_REFINE', $config['max_num_search_keywords'], $num_keywords));
		}

		// $keywords input format: each word separated by a space, words in a bracket are not separated

		// the user wants to search for any word, convert the search query
		if ($terms == 'any')
		{
			$words = array();

			preg_match_all('#([^\\s+\\-|()]+)(?:$|[\\s+\\-|()])#u', $keywords, $words);
			if (sizeof($words[1]))
			{
				$keywords = '(' . implode('|', $words[1]) . ')';
			}
		}

		// set the search_query which is shown to the user
		$this->search_query = $keywords;

		$exact_words = array();
		preg_match_all('#([^\\s+\\-|*()]+)(?:$|[\\s+\\-|()])#u', $keywords, $exact_words);
		$exact_words = $exact_words[1];

		$common_ids = $words = array();

		if (sizeof($exact_words))
		{
			$word_id = 1;
			foreach ($exact_words as $exact_word)
			{
				$words[$exact_word] = $exact_word; //(int) $word_id;
				$word_id++;
			}
		/**
			$sql = 'SELECT word_id, word_text, word_common
				FROM ' . SEARCH_WORDLIST_TABLE . '
				WHERE ' . $db->sql_in_set('word_text', $exact_words) . '
				ORDER BY word_count ASC';
			$result = $db->sql_query($sql);

			// store an array of words and ids, remove common words
			while ($row = $db->sql_fetchrow($result))
			{
				if ($row['word_common'])
				{
					$this->common_words[] = $row['word_text'];
					$common_ids[$row['word_text']] = (int) $row['word_id'];
					continue;
				}

				$words[$row['word_text']] = (int) $row['word_id'];
			}
			$db->sql_freeresult($result);
		**/
		}
		unset($exact_words);

		// now analyse the search query, first split it using the spaces
		$query = explode(' ', $keywords);

		$this->must_contain_ids = array();
		$this->must_not_contain_ids = array();
		$this->must_exclude_one_ids = array();

		$mode = '';
		$ignore_no_id = true;

		foreach ($query as $word)
		{
			if (empty($word))
			{
				continue;
			}

			// words which should not be included
			if ($word[0] == '-')
			{
				$word = substr($word, 1);

				// a group of which at least one may not be in the resulting posts
				if ($word[0] == '(')
				{
					$word = array_unique(explode('|', substr($word, 1, -1)));
					$mode = 'must_exclude_one';
				}
				// one word which should not be in the resulting posts
				else
				{
					$mode = 'must_not_contain';
				}
				$ignore_no_id = true;
			}
			// words which have to be included
			else
			{
				// no prefix is the same as a +prefix
				if ($word[0] == '+')
				{
					$word = substr($word, 1);
				}

				// a group of words of which at least one word should be in every resulting post
				if ($word[0] == '(')
				{
					$word = array_unique(explode('|', substr($word, 1, -1)));
				}
				$ignore_no_id = false;
				$mode = 'must_contain';
			}

			if (empty($word))
			{
				continue;
			}

			// if this is an array of words then retrieve an id for each
			if (is_array($word))
			{
				$non_common_words = array();
				$id_words = array();
				foreach ($word as $i => $word_part)
				{
					if (strpos($word_part, '*') !== false)
					{
						$id_words[] = '\'' . $db->sql_escape(str_replace('*', '%', $word_part)) . '\'';
						$non_common_words[] = $word_part;
					}
					else if (isset($words[$word_part]))
					{
						$id_words[] = $words[$word_part];
						$non_common_words[] = $word_part;
					}
					else
					{
						$len = utf8_strlen($word_part);
						if ($len < $this->word_length['min'] || $len > $this->word_length['max'])
						{
							$this->common_words[] = $word_part;
						}
					}
				}
				if (sizeof($id_words))
				{
					sort($id_words);
					if (sizeof($id_words) > 1)
					{
						$this->{$mode . '_ids'}[] = $id_words;
					}
					else
					{
						$mode = ($mode == 'must_exclude_one') ? 'must_not_contain' : $mode;
						$this->{$mode . '_ids'}[] = $id_words[0];
					}
				}
				// throw an error if we shall not ignore unexistant words
				else if (!$ignore_no_id && sizeof($non_common_words))
				{
					trigger_error(sprintf($user->lang['WORDS_IN_NO_POST'], implode(', ', $non_common_words)));
				}
				unset($non_common_words);
			}
			// else we only need one id
			else if (($wildcard = strpos($word, '*') !== false) || isset($words[$word]))
			{
				if ($wildcard)
				{
					$len = utf8_strlen(str_replace('*', '', $word));
					if ($len >= $this->word_length['min'] && $len <= $this->word_length['max'])
					{
						$this->{$mode . '_ids'}[] = '\'' . $db->sql_escape(str_replace('*', '%', $word)) . '\'';
					}
					else
					{
						$this->common_words[] = $word;
					}
				}
				else
				{
					$this->{$mode . '_ids'}[] = $words[$word];
				}
			}
			// throw an error if we shall not ignore unexistant words
			else if (!$ignore_no_id)
			{
				if (!isset($common_ids[$word]))
				{
					$len = utf8_strlen($word);
					if ($len >= $this->word_length['min'] && $len <= $this->word_length['max'])
					{
						trigger_error(sprintf($user->lang['WORD_IN_NO_POST'], $word));
					}
					else
					{
						$this->common_words[] = $word;
					}
				}
			}
			else
			{
				$len = utf8_strlen($word);
				if ($len < $this->word_length['min'] || $len > $this->word_length['max'])
				{
					$this->common_words[] = $word;
				}
			}
		}
/**
#print_r("query=(");					print_r($query);						print_r(")<hr />");
#print_r("ignore_no_id=(");			print_r($ignore_no_id);					print_r(")<hr />");
#print_r("common_ids=(");			print_r($common_ids);					print_r(")<hr />");

#print_r("search_query=(");			print_r($this->search_query);			print_r(")<hr />");
 print_r("must_contain_ids=(");		print_r($this->must_contain_ids);		print_r(")<hr />");
 print_r("must_not_contain_ids=(");	print_r($this->must_not_contain_ids);	print_r(")<hr />");
 print_r("must_exclude_one_ids=(");	print_r($this->must_exclude_one_ids);	print_r(")<hr />");
#print_r("common_words=(");			print_r($this->common_words);			print_r(")<hr />");
**/
		// we can't search for negatives only
		if (!sizeof($this->must_contain_ids))
		{
			return false;
		}

		if (!empty($this->search_query))
		{
			return true;
		}
		return false;
	}
	/**
	* Performs a search on keywords depending on display specific params. You have to run split_keywords() first.
	*
	* @param	string		$type				contains either posts or topics depending on what should be searched for
	* @param	string		$fields				contains either titleonly (topic titles should be searched), msgonly (only message bodies should be searched), firstpost (only subject and body of the first post should be searched) or all (all post bodies and subjects should be searched)
	* @param	string		$terms				is either 'all' (use query as entered, words without prefix should default to "have to be in field") or 'any' (ignore search query parts and just return all posts that contain any of the specified words)
	* @param	array		$sort_by_sql		contains SQL code for the ORDER BY part of a query
	* @param	string		$sort_key			is the key of $sort_by_sql for the selected sorting
	* @param	string		$sort_dir			is either a or d representing ASC and DESC
	* @param	string		$sort_days			specifies the maximum amount of days a post may be old
	* @param	array		$ex_fid_ary			specifies an array of forum ids which should not be searched
	* @param	array		$m_approve_fid_ary	specifies an array of forum ids in which the searcher is allowed to view unapproved posts
	* @param	int			$topic_id			is set to 0 or a topic id, if it is not 0 then only posts in this topic should be searched
	* @param	array		$author_ary			an array of author ids if the author should be ignored during the search the array is empty
	* @param	string		$author_name		specifies the author match, when ANONYMOUS is also a search-match
	* @param	array		&$id_ary			passed by reference, to be filled with ids for the page specified by $start and $per_page, should be ordered
	* @param	int			$start				indicates the first index of the page
	* @param	int			$per_page			number of ids each page is supposed to contain
	* @return	boolean|int						total number of results
	*
	* @access	public
	*/
	function keyword_search_book($text = '', $file = '')
	{
		// No keywords? No books.
		if (empty($this->search_query) || $text == '')
		{
			return false;
		}

		$text = strtolower($text);
		$return = false;
		foreach ($this->must_contain_ids as $subquery)
		{
			if (is_array($subquery))
			{
				foreach ($subquery as $id)
				{
					$id = strtolower($id);
//					$return = strpos($text, $id);
					if (strpos($text, $id) !== false)
					{
						$return = true;
					}
				}
			}
			else
			{
				$subquery = strtolower($subquery);
//				$return = strpos($text, $subquery);
				if (strpos($text, $subquery) !== false)
				{
					$return = true;
				}
			}
			/**
			$matches = array();
			preg_match_all('/' . trim($subquery) . '/i', $text, $matches);
			print_r($matches);
			if (sizeof($matches) && sizeof($matches[0]))
			{
				$return = true;
			}
			print_r("<br /><b>must_contain_ids</b> subquery=($subquery) book_file=($file) || return=(".(($return)?'true':'false').")<br />");
			**/
		}

		foreach ($this->must_not_contain_ids as $subquery)
		{
			$subquery = strtolower($subquery);
//			$return = strpos($text, $subquery);
			if (strpos($text, $subquery) !== false)
			{
				$return = false;
			}
			/**
			$matches = array();
			preg_match_all('/' . trim($subquery) . '/i', $text, $matches);
			if (sizeof($matches) && sizeof($matches[0]))
			{
				$return = false;
			}
			print_r("<br /><b>must_not_contain_ids</b> subquery=($subquery) book_file=($file) || return=(".(($return)?'true':'false').")<br />");
			**/
		}

		foreach ($this->must_exclude_one_ids as $subquery)
		{
			$subquery = strtolower($subquery);
//			$return = strpos($text, $subquery);
			if (strpos($text, $subquery) !== false)
			{
				$return = true;
			}
			/**
			$matches = array();
			preg_match_all('/' . trim($subquery) . '/siU', $text, $matches);
			if (sizeof($matches) && sizeof($matches[0]))
			{
				$return = true;
			}
			print_r("<br /><b>must_exclude_one_ids</b> subquery=($subquery) book_file=($file) || return=(".(($return)?'true':'false').")<br />");
			**/
		}
		return $return;
	}
}

?>